;; Copyright 2007, 2009, 2010, 2011, 2013, 2015, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

;;-----------------

;; width=100% leave alone ...


(progn
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (autoload 'html-sizes-update "html-sizes" nil t)

  (setq html-sizes-link-table
        '(("ftp://foo.com/bar/" . "/etc/")))

  (find-file "d-html-sizes.html")
  (html-sizes-update))

(setq html-sizes-update-query nil)
(setq html-sizes-update-query t)


(string-match "\.\\(asc\\|c\\|el\\|txt\\)"))
(html-sizes-text-filename-p "foo.dz")
(html-sizes-text-filename-p "foo.txt.gz")
(html-sizes-text-filename-p "foo.tar.gz")
(html-sizes-text-filename-p "foo.tar.Z")
(html-sizes-text-filename-p "foo.tar.bz2")
(html-sizes-text-filename-p "foo.txt")
(html-sizes-text-filename-p "foo.deb")
(html-sizes-text-filename-p "foo.utf8")
(html-sizes-text-filename-p "foo")
(find-operation-coding-system 'insert-file-contents "foo.txt")
(find-operation-coding-system 'insert-file-contents "foo.utf8")
(find-operation-coding-system 'insert-file-contents "foo.c")
(find-operation-coding-system 'insert-file-contents "foo.tar")
(find-operation-coding-system 'insert-file-contents "foo.deb")

(html-sizes-file-charset buffer-file-name)
(html-sizes-file-charset "/etc/motd")
(html-sizes-file-charset (concat "../../xti""de/xt""ide.el"))
(html-sizes-file-charset "/usr/share/doc/doc-base/doc-base.txt.gz")


;;-----------------------------------------------------------------------------
;; content-type override vs extension

https://developer.mozilla.org/en/How_Mozilla_determines_MIME_Types

;;-----------------------------------------------------------------------------
;; image-size

(setq foo (create-image "/tmp/screenshot.png"))
(let ((default-directory "/tmp"))
  (setq foo (create-image (expand-file-name "screenshot.png")))
  (image-size foo t))
(let ((default-directory "/tmp"))
  (setq foo (create-image (identity "x.png")))
  (image-size foo t))
(setq foo (create-image "/tmp/x.png"))



;;-----------------------------------------------------------------------------
;; perform-replace

(perform-replace "\\=foo"  ;; old text
                 "bar"  ;; new text
                 nil   ;; query
                 t      ;; regexp
                 nil   ;; don't demand word boundary
                 nil   ;; no repeat-count business
                 nil)   ;; standard keymap
foo

foo

;;-----------------------------------------------------------------------------


(regexp-opt (mapcar 'char-to-string (charset-chars 'ascii)))

(defun html-sizes-any-nonascii-p ()
  "Return non-nil if there's any non-ASCII characters in the current buffer."
  (save-excursion
    (goto-char (point-min))
    (skip-chars-forward "\000-\177")
    (/= (point) (point-max))))

(defun html-sizes-file-charset (filename)
  (with-temp-buffer
    (with-current-buffer (or (get-file-buffer filename)
                             (progn
                               (condition-case nil
                                   ;; empty buffer if no such file
                                   (insert-file-contents filename)
                                 (error nil))
                               (current-buffer)))
      (and (html-sizes-any-nonascii-p)
           (coding-system-get buffer-file-coding-system 'mime-charset)))))

(coding-system-get 'utf-8 'mime-charset)
(coding-system-get 'iso-latin-1 'mime-charset)
(coding-system-get 'iso-latin-1-unix 'mime-charset)
(html-sizes-file-charset "../tty-format.el")

(defun html-sizes-update-charset ()
  (interactive)
  (let ((total   0)
        (differ  0)
        (missing 0)
        (quit    nil))
    (save-excursion
      (goto-char (point-min))
      (while (and (not quit)
                  (re-search-forward "<a\\(\\s-\\)\\(.\\|\n\\)*?href=\"\\([^\"\n]+\\)\">\\(.\\|\n\\)*?>" nil t))
        (let* ((charset-beg (match-beginning 1))
               (charset-end (match-end 1))
               (got-charset (match-string 1))
               (filename    (match-string 3))
               (prefix      got-charset))

          (save-excursion
            (save-restriction
              (narrow-to-region (match-beginning 0) (match-end 0))
              (goto-char (point-min))
              (when (re-search-forward "charset=\"[^\"]*\"" nil t)
                (setq charset-beg (match-beginning 0)
                      charset-end (match-end 0)
                      got-charset (match-string 0)
                      prefix      ""))))

          (when (string-match "\\`ftp:" filename)

            (dolist (elem html-sizes-link-table)
              (when (string-match (concat "\\`" (regexp-quote (car elem)))
                                  filename)
                (setq filename (concat (cdr elem)
                                       (substring filename (match-end 0))))))


            (let ((want-charset (html-sizes-file-charset filename)))
              (setq want-charset (if want-charset
                                     (format "%scharset=\"%s\""
                                             prefix want-charset)
                                   got-charset))
              (unless (equal got-charset want-charset)
                (setq differ (1+ differ))

                (or (perform-replace got-charset   ;; old text
                                     want-charset  ;; new text
                                     html-sizes-update-query
                                     nil  ;; not a regexp
                                     nil  ;; don't demand word boundary
                                     nil  ;; no repeat-count business
                                     nil  ;; standard keymap
                                     charset-beg  ;; only this
                                     charset-end) ;; single change
                    (goto-char (point-max)))))))))

    (message "Total %d file sizes, %d differed, %d missing"
             total differ missing)))


(coding-system-get 'undecided 'mime)


;;-----------------
;; IHDR not necessarily at start of file

    (insert-file-contents-literally filename nil 0 12)
    ;;                              1  2           3           4           5              6
    (goto-char (point-min))
    (debug)
    (and (= 12 (- (point-max) (point-min)))
         (looking-at "IHDR")
         (list (html-sizes-u32be-to-number
                (buffer-substring (+ 4 (point-min))    ;; width
                                  (+ 8 (point-min))))
               (html-sizes-u32be-to-number
                (buffer-substring (+ 8 (point-min))    ;; height
                                  (+ 12 (point-min))))))


;;-----------------
;; perform-replace

(perform-replace "\\="  ;; old
                 "blah" ;; new
                 t      ;; query
                 t      ;; regexp
                 nil)   ;; don't demand word boundary
blahfblahjblahsblahdkjf sdkf s


;;-----------------
;; perform-replace

html-sizes-link-table
(html-sizes-link-apply "http://download.tuxfamily.org/user42/triples/triples.ps")
(html-sizes-link-apply "http://download.tuxfamily.org/user42/triples.ps")


;;-----------------
;; doc-view run

(debug-on-entry 'doc-view-mode)
(cancel-debug-on-entry 'doc-view-mode)
(html-sizes-text-filename-p "foo.pdf")
(find-operation-coding-system 'insert-file-contents "foo.pdf")
(find-operation-coding-system 'insert-file-contents "foo.ps")

;;-----------------------------------------------------------------------------

(html-sizes-file-size-string "/etc/passwd" 2 ?b "Bytes")
(html-sizes-file-size-string "/etc/passwd" 0 ?k "xx")
(html-sizes-file-size-string "/etc/passwd" 1 ?k "xx")
(html-sizes-file-size-string "/etc/passwd" 2 ?k "xx")
(html-sizes-file-size-string "/etc/passwd" 3 ?k "xx")
(html-sizes-file-size-string "/etc/passwd" 6 ?m "mbytes")

(html-sizes-file-size-string "/usr/bin/gdb" 2 ?b "Bytes")
(html-sizes-file-size-string "/usr/bin/gdb" 0 ?k "xx")
(html-sizes-file-size-string "/usr/bin/gdb" 1 ?k "xx")
(html-sizes-file-size-string "/usr/bin/gdb" 2 ?k "xx")
(html-sizes-file-size-string "/usr/bin/gdb" 3 ?k "xx")
(html-sizes-file-size-string "/usr/bin/gdb" 1 ?m "mbytes")
(html-sizes-file-size-string "/usr/bin/gdb" 6 ?m "mbytes")

;;-----------------------------------------------------------------------------

perl -MImage::ExifTool=:Public -e 'print values(%{ImageInfo($ARGV[0],['PageCount'])})' /usr/share/doc/texlive-doc/plain/impatient/book.pdfx


(html-sizes-file-pages "/usr/share/doc/texlive-doc/plain/impatient/book.pdf")
(html-sizes-file-pages "/nosuchfile")
(html-sizes-file-pages "/etc/passwd")
