#!/usr/bin/perl -w

# Copyright 2009, 2010 Kevin Ryde.
#
# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.

use 5.008;
use strict;
use warnings;
use DBI;
use DBD::SQLite 1.27;
use File::Spec;

use App::Chart;

use constant DEBUG => 0;

my $filename = 'd-sqlite-dump-utf8.sqdb';
unlink $filename;
my $dbh = DBI->connect ("dbi:SQLite:dbname=$filename",
                        '', '', {RaiseError=>1});
$dbh->func(90_000, 'busy_timeout');  # 90 seconds
$dbh->{'sqlite_unicode'} = 1;

{
  $dbh->do (<<'HERE');
CREATE TABLE foo (
    key     TEXT     NOT NULL       PRIMARY KEY,
    value   TEXT     DEFAULT NULL
    )
HERE

  my $key = "bullet";
  my $value = "char \x{2022} bullet";
  $dbh->do ('INSERT INTO foo (key,value) VALUES (?,?)',
            undef,
            $key,$value);
}
{
  $dbh->do (<<'HERE');
CREATE TABLE blobs (
    key     TEXT     NOT NULL       PRIMARY KEY,
    value   BLOB     DEFAULT NULL
    )
HERE

  {
    my $key = "blob1";
    my $value = join('',map {chr} reverse(0 .. 255));

    my $sth = $dbh->prepare_cached
      ('INSERT INTO blobs (key,value) VALUES (?,?)');
    $sth->bind_param (1, $key);
    $sth->bind_param (2, $value, DBI::SQL_BLOB());
    $sth->execute;
    $sth->finish;
  }
  {
    my $key = "blob2";
    my $value = "abc\0def";

    my $sth = $dbh->prepare_cached
      ('INSERT INTO blobs (key,value) VALUES (?,?)');
    $sth->bind_param (1, $key);
    $sth->bind_param (2, $value, DBI::SQL_BLOB());
    $sth->execute;
    $sth->finish;
  }
}

$dbh->disconnect;
