;; Copyright 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look for unrecognised backslash \ forms in docstrings.
;; The recognised forms are per "(elisp)Keys in Documentation"
;;     \[   command key sequence
;;     \{   keymap variable
;;     \<   keymap variable for key sequence
;;     \=   literal next character, such as \=\

(defun my-docstring-backslashes ()
  (interactive)

  (require 'info-xref)
  (pop-to-buffer info-xref-output-buffer t)
  (let ((info-xref-filename-heading "")
	(case-fold-search t))

    (info-xref-output "Checking docstring backslashes ...\n")
    (with-temp-buffer
      (mapatoms
       (lambda (symbol)
         (let ((str (condition-case nil
                        (documentation symbol
                                       t) ;; raw
                      (error nil))))
           (when str
             (erase-buffer)
             (insert str)
             (goto-char (point-min))
             (while (re-search-forward "\\\\\\(\\([^[{<=]\\)\\|=\\(.\\|\n\\)?\\)" nil t)
               (when (match-beginning 2)
                 (info-xref-output "%s\n  \\%s\n"
                                   symbol (match-string 0)))))))))
    (info-xref-output "done\n")))

(my-load-all--autoloads)
(my-docstring-backslashes)
