;; Copyright 2009, 2010, 2015 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  ;; xemacs 21.4.22 in -batch mode segfaults on something to do with window
  ;; size, so dummy out this shrink
  (defun shrink-window-if-larger-than-buffer (&optional window))

  (let ((count 0)
        (case-fold-search nil)
        (tempbuf "*t-checkdoc-batch.el temp*"))
    (switch-to-buffer tempbuf)
    (erase-buffer)


    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(checkdoc-batch))

    (mapcar (lambda (func)
              (unless (commandp func)
                (error "`%S' not interactive" func))
              (setq count (1+ count)))
            '(checkdoc-batch))

    ;;-------------------------------------------------------------------------
    ;; checkdoc-batch

    (message "checkdoc-batch first")
    (checkdoc-batch)
    (message "checkdoc-batch second")
    (checkdoc-batch)


    ;;-------------------------------------------------------------------------
    ;; checkdoc-batch--with-buffer-file-name

    (require 'checkdoc-batch)
    (if (not (fboundp 'checkdoc-batch--with-buffer-file-name))
        (message "skip, no `checkdoc-batch--with-buffer-file-name' macro after byte compiled")

      (with-temp-buffer
        (let ((workbuf (current-buffer)))
          (setq buffer-file-name "/tmp/t-checkdoc-batch/one.txt")

          (checkdoc-batch--with-buffer-file-name "/tmp/t-checkdoc-batch/two.txt"
            (unless (equal (buffer-file-name) "/tmp/t-checkdoc-batch/two.txt")
              (error "checkdoc-batch--with-buffer-file-name two.txt not set"))
            (set-buffer tempbuf))

          (set-buffer workbuf)
          (unless (equal (buffer-file-name) "/tmp/t-checkdoc-batch/one.txt")
            (error "checkdoc-batch--with-buffer-file-name one.txt not restored, got %S" (buffer-file-name)))

          (checkdoc-batch--with-buffer-file-name "/tmp/t-checkdoc-batch/three.txt"
            (kill-buffer (current-buffer)))

          (set-buffer tempbuf)
          (unless (equal (buffer-file-name) nil)
            (error "checkdoc-batch--with-buffer-file-name tempbuf not still nil"))
          (setq count (1+ count)))))


    ;;-------------------------------------------------------------------------
    ;; checkdoc-batch-linenum

    (require 'checkdoc-batch)
    (erase-buffer)
    (checkdoc-batch-linenum (point-min))
    (setq count (1+ count))

    (insert "foo")
    (checkdoc-batch-linenum (point-min))
    (checkdoc-batch-linenum (point-max))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (require 'checkdoc-batch)
    (unload-feature 'checkdoc-batch t)


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- no advice

    (require 'checkdoc-batch)

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'checkdoc-batch))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'checkdoc-batch))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
