;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(compilation-recenter-end-enable))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((compilation-mode-hook compilation-recenter-end-enable)))


    ;;-------------------------------------------------------------------------
    ;; compilation-recenter-end-enable

    (compilation-recenter-end-enable)
    (when (boundp 'compilation-finish-functions) ;; emacs
      (unless (or (eq 'compilation-recenter-end-at-finish
                      compilation-finish-functions)
                  (and (listp compilation-finish-functions)
                       (memq 'compilation-recenter-end-at-finish
                             compilation-finish-functions)))
        (error "oops, compilation-finish-functions not set")))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")
      (if (not (boundp 'compilation-finish-functions))
          (message "skip, no `compilation-finish-functions'")

        (unload-feature 'compilation-recenter-end t)

        (when (memq 'compilation-recenter-end-at-finish
                    compilation-finish-functions)
          (error "unload-feature leftover in compilation-finish-functions"))
        (setq count (1+ count))))

    ;; couple of cycles to exercise
    (require 'compilation-recenter-end)
    (unload-feature 'compilation-recenter-end t)
    (require 'compilation-recenter-end)
    (unload-feature 'compilation-recenter-end t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

