;; Copyright 2009, 2010, 2013 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-in-first-line-p ()
  "Return non-nil if point is the first line of the buffer."
  (save-excursion
    (not (search-backward "\n" nil t))))

(defun my-run-test ()

  (let ((count 0)
        (tempbuf "*t-ffap-include-start--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'ffap)
    (require 'ffap-include-start)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (boundp 'byte-compile-unresolved-functions)
      (setq byte-compile-unresolved-functions
            (delete '(thing-at-point-looking-at 1) ;; loaded dynamically
                    byte-compile-unresolved-functions))
      (when byte-compile-unresolved-functions
        (message "byte-compile-unresolved-functions is %S"
                 byte-compile-unresolved-functions)
        ;; (message "fb %S" (fboundp (caar byte-compile-unresolved-functions)))
        (error "byte-compile-unresolved-functions after load: %S"
               byte-compile-unresolved-functions)))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point

    (mapcar (lambda (elem)
              (let ((str   (nth 0 elem))
                    (want1 (nth 1 elem))
                    (want2 (nth 2 elem)))
                (erase-buffer)
                (insert str)

                (goto-char (point-min))
                (while (not (eobp))
                  (setq ffap-string-at-point "dummy value")
                  (let ((want (if (my-in-first-line-p) want1 want2))
                        (got  (ffap-string-at-point 'file)))
                    (unless (equal want got)
                      (error "got %S want %S on %S" got want str))
                    (unless (equal got ffap-string-at-point)
                      (error "got %S not put into ffap-string-at-point" got)))
                  (forward-char)))
              (setq count (1+ count)))

            '(("#include <foo.h>"        "foo.h")
              ("#include <foo.h\n"       "foo.h")  ;; unclosed angle
              ("#include <foo.h"         "foo.h")  ;; unclosed angle
              ("#\tinclude\t<foo.h>"     "foo.h")  ;; tabs
              ("#include \"bar/quux.h\"" "bar/quux.h")
              ("#include \"abc.h"        "abc.h")  ;; unclosed quote
              ("#include \"abc.h\n"      "abc.h")  ;; unclosed quote
              ("#include <abc\n/def.h>"  "abc" "/def.h")

              ("include foo.make"  "foo.make")
              ("include foo.make\nblah"  "foo.make" "blah")

              ("include \"foo.rc\""  "foo.rc")
              ("include \"foo.rc\n"  "foo.rc")
              ("include \"foo.rc"  "foo.rc")

              ("@include \"foo.gawk\""  "foo.gawk")
              ("@ \tinclude \"foo.gawk\""  "foo.gawk")
              ("@include \"foo.gawk\n"  "foo.gawk")
              ("@include \"foo.gawk"  "foo.gawk") ;; unclosed quote
              ))


    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point

    (mapcar (lambda (elem)
              (let ((str   (nth 0 elem))
                    (want1 (nth 1 elem))
                    (want2 (nth 2 elem)))
                (erase-buffer)
                (insert str)

                (goto-char (point-min))
                (setq ffap-string-at-point "dummy value")
                (let ((want want1)
                      (got  (ffap-string-at-point 'file)))
                  (unless (equal want got)
                    (error "got %S want %S" got want))
                  (unless (equal got ffap-string-at-point)
                    (error "got %S not put into ffap-string-at-point" got)))

                (goto-char (point-max))
                (setq ffap-string-at-point "dummy value")
                (let ((want want2)
                      (got  (ffap-string-at-point 'file)))
                  (unless (equal want got)
                    (error "got %S want %S" got want))
                  (unless (equal got ffap-string-at-point)
                    (error "got %S not put into ffap-string-at-point" got))))

              (setq count (1+ count)))

            '(("include foo.make bar.make"  "foo.make" "bar.make")
              ))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'ffap-include-start))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'ffap-include-start))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
