;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (function-list '(info-lookmore-elisp-userlast
                         info-lookmore-elisp-cl
                         info-lookmore-elisp-gnus
                         info-lookmore-apropos-elisp
                         info-lookmore-c-gcc
                         info-lookmore-c-readline
                         info-lookmore-c++-use-c
                         info-lookmore-coreutils-index)))

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            function-list)


    ;;-------------------------------------------------------------------------
    ;; each function at least runs ...

    (mapcar (lambda (symbol)
              (funcall symbol)
              (setq count (1+ count)))
            function-list)

    (message "have sh-mode docs: %S"
             (not (not (info-lookup->doc-spec 'symbol 'sh-mode))))


    ;;-------------------------------------------------------------------------
    ;; `info-lookmore-elisp-gnus' emacs-mime TRANS-FUNC operation

    (progn
      (info-lookmore-elisp-gnus)
      (let* ((doc-spec   (info-lookup->doc-spec 'symbol 'emacs-lisp-mode))
             (doc        (or (assoc "(emacs-mime)Index" doc-spec)
                             (error "Oops, (emacs-mime) doc not found")))
             (trans-func (nth 1 doc)))

        (dolist (elem '(("mm-foo" "mm-foo")
                        ("LaTeX"  nil)))
          (let* ((item (nth 0 elem))
                 (want (nth 1 elem))
                 (got  (funcall trans-func item)))
            (unless (equal want got)
              (error "item %S got %S want %S" item got want))
            (setq count (1+ count))))))


    ;;-------------------------------------------------------------------------
    ;; `info-lookmore-c-gcc' TRANS-FUNC operation

    (progn
      (info-lookmore-c-gcc)
      (let* ((doc-spec   (info-lookup->doc-spec 'symbol 'c-mode))
             (doc        (or (assoc "(gcc)Keyword Index" doc-spec)
                             (error "Oops, (gcc) doc not found")))
             (trans-func (nth 1 doc)))

        (dolist (elem '(("__builtin_expect"     "__builtin_expect")
                        ("__float128 data type" "__float128")
                        ("__func__ identifier"  "__func__")
                        ("__complex__ keyword"  "__complex__")
                        ("declaration scope"    nil)))
          (let* ((item (nth 0 elem))
                 (want (nth 1 elem))
                 (got  (funcall trans-func item)))
            (unless (equal want got)
              (error "item %S got %S want %S" item got want))
            (setq count (1+ count))))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'info-lookmore)


    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
