;; Copyright 2009, 2010, 2011 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-nobreak-fade--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoload fboundp" func))
              (setq count (1+ count)))
            '(nobreak-fade-single-letter-p
              nobreak-fade-emacs-url-p
              nobreak-fade-emacs-M-x-p
              nobreak-fade-emacs-info-link-p))

    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-add

    (require 'nobreak-fade)
    (mapcar (lambda (elem)
              (let* ((before (nth 0 elem))
                     (pred   (nth 1 elem))
                     (want   (nth 2 elem)))
                (setq fill-nobreak-predicate before)
                (nobreak-fade-add pred)
                (let ((got fill-nobreak-predicate))
                  (unless (equal got want)
                    (error "before %S\ngot  %S\nwant %S" before got want))
                  (setq count (1+ count)))))

            ;; (put 'fill-nobreak-predicate 'custom-type nil)
            (cond ((eq 'hook (get 'fill-nobreak-predicate 'custom-type))
                   '((nil foo (foo))
                     ((bar) foo (foo bar))
                     ((foo) foo (foo))))

                  ((boundp 'fill-nobreak-predicate)
                   '((nil foo foo)
                     (foo foo foo)

                     (bar
                      foo
                      (lambda () (or (foo) (bar))))

                     ((lambda () (or (bar)))
                      foo
                      (lambda () (or (foo) (bar))))

                     ((lambda () (or (bar) (quux)))
                      foo
                      (lambda () (or (foo) (bar) (quux))))

                     (123
                      foo
                      (lambda () (or (foo) (123))))
                     ([123]
                      foo
                      (lambda () (or (foo) ([123]))))
                     ((xxx yyy)
                      foo
                      (lambda () (or (foo) ((xxx yyy)))))
                     ))))

    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-single-letter-p

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (pos  (nth 1 elem))
                     (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (not (not (nobreak-fade-single-letter-p)))))
                  (unless (equal want got)
                    (error "str %S got %S want %S" str got want))
                  (setq count (1+ count)))))

            '(("A blah" 2 t)
              ("`A blah" 3 t)
              ("A, blah" 3 t)
              ("(A blah" 3 t)
              ("[A blah" 3 t)
              ("A) blah" 3 t)
              ("(A) blah" 4 nil) ;; currently

              ("Blah I" 5 t)
              ("Blah I." 5 t)
              ("Blah I'." 5 t)
              ("Blah I\n\n" 5 t)
              ("Blah I)" 5 t)
              ("Blah I)." 5 t)
              ("Blah I}}." 5 nil)))

    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-emacs-url-p

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (pos  (nth 1 elem))
                     (want (nth 2 elem))
                     (case-fold-search t))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (not (not (nobreak-fade-emacs-url-p)))))
                  (unless (equal want got)
                    (error "str %S got %S want %S" str got want))
                  (setq count (1+ count)))))

            '(
              (" url `http"  5 nil)
              (" URL or something" 5 nil)
              ("URL, `http"  5 nil)
              (" URL `http"  5 t)
              ("CURL `http"  5 nil)))


    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-emacs-M-x-p

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (pos  (nth 1 elem))
                     (want (nth 2 elem))
                     (case-fold-search t))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (not (not (nobreak-fade-emacs-M-x-p)))))
                  (unless (equal want got)
                    (error "str %S got %S want %S" str got want))
                  (setq count (1+ count)))))

            '((""        0 nil)
              ("M x"     2 nil)
              ("M- x"     3 nil)
              ("M- x"     3 nil)
              ("  M-x foo"     6 t)
              ("  M-x `"       6 nil)
              ("  M-x --"      6 nil)
              ("  M-X mail"    6 nil)
              (" `M-x foo'"    6 t)
              ("`M-x' or foo'" 6 nil)
              ("  M-x foo'"    6 t)
              ("IBM-x foo'"    6 nil)
              ("IBM-x foo'"    6 nil)))


    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-emacs-info-link-p

    (if (featurep 'xemacs)
        (message "skip, no help-xref-info-regexp in xemacs21")

      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (pos  (nth 1 elem))
                       (want (nth 2 elem))
                       (case-fold-search t))
                  (erase-buffer)
                  (insert str)
                  (goto-char (+ (point-min) pos))
                  (let ((got (not (not (nobreak-fade-emacs-info-link-p)))))
                    (unless (equal want got)
                      (error "str %S got %S want %S" str got want))
                    (setq count (1+ count)))))

              '((""          0 nil)
                ("Info node" 5 nil)
                ("Info node `(elisp)Documentation Tips'"
                 32 t)
                ("Blah node `(elisp)Documentation Tips'"
                 32 nil))))

    ;;-------------------------------------------------------------------------
    ;; nobreak-fade-completing-read

    (let ((orig-func (symbol-function 'completing-read))
          got)
      (unwind-protect
          (progn
            (fset 'completing-read (lambda (&rest args) "car"))
            (setq got (nobreak-fade-completing-read '(car cdr) nil)))
        (fset 'completing-read orig-func))

      (let ((want 'car))
        (unless (equal got want)
          (error "nobreak-fade-completing-read got %S want %S" got want))))


    ;;-------------------------------------------------------------------------
    ;; unload

    (unload-feature 'nobreak-fade t)


    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
