;; Copyright 2006, 2007, 2008, 2009, 2010, 2011 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defconst my-have-org-mode-p
  (condition-case nil
      (require 'org)
    (error nil)))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count   0)
        (tempbuf "*t-tex-math-preview--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(tex-math-preview))

    ;;-------------------------------------------------------------------------
    ;; customize

    (require 'tex-math-preview) ;; not autoloaded in emacs21
    (save-excursion
      (customize-group 'tex-math-preview)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; risky-local-variable-p

    (if (not (fboundp 'risky-local-variable-p))
        (message "skip, no `risky-local-variable-p'")

      (mapcar (lambda (symbol)
                (unless (boundp symbol)
                  (error "oops, no variable: %S" symbol))
                (unless (risky-local-variable-p symbol (symbol-value symbol))
                  (error "%S not `risky-local-variable-p'" symbol))
                (setq count (1+ count)))
              '(tex-math-preview-function)))


    ;;-------------------------------------------------------------------------
    ;; derived-mode-p

    ;; check derived-mode-p is available for use by tex-math-preview-str
    ;; (no separate tex/latex decision func as yet)
    ;;
    (with-temp-buffer
      (fundamental-mode)
      (unless (derived-mode-p 'fundamental-mode)
        (error "Oops, derived-mode-p didn't recognise fundamental-mode"))
      (setq count (1+ count))

      (when (derived-mode-p 'no-such-mode)
        (error "Oops, derived-mode-p true for 'no-such-mode"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; thing-at-point 'tex-math

    (require 'tex-math-preview)

    (mapcar (lambda (elem)
              (let ((end  (make-marker))
                    (pre  (nth 0 elem))
                    (str  (nth 1 elem))
                    (post (nth 2 elem))
                    (want (nth 3 elem)))
                (erase-buffer)
                (insert pre)
                (save-excursion
                  (insert str)
                  (set-marker end (point-marker))
                  (insert post))

                (while (< (point) end)
                  (let ((got (thing-at-point 'tex-math)))
                    (unless (equal got want)
                      (error "Got \"%s\" want \"%s\"" got want)))
                  (setq count (1+ count))
                  (forward-char))))

            '(;; $...$
              ;;
              ("$"       " 1 + 2 "  "$"           " 1 + 2 ")
              ("$"       " \\$+\\$" "$"           " \\$+\\$")
              ("$"       "1\n+\n2"  "$"           "1\n+\n2")
              ("$"       "1+2"      "$   $"       "1+2")
              ("$"       "1+2"      "$\n$3+4$"    "1+2")
              ("$3+4$ $" "1+2"      "$\n$3+4$"    "1+2")
              ("$3+4 $"  "1+2"      "$\n$"        "1+2")
              ("<math>" "$1+2"      "$</math>"    "1+2")
              ("@math{" "$1+2"      "$}"          "1+2")

              ;; $$...$$
              ;;
              ("$"       "$1+2"          "$$"          "1+2")
              ("$"       "$\n1\n+\n2\n"  "$$"          "\n1\n+\n2\n")
              ("$3+4 $"  "$1+2"          "$$\n$"       "1+2")
              ("<math>$" "$1+2"          "$$</math>"   "1+2")
              ("@math{$" "$1+2"          "$$}"         "1+2")

              ;; <math>...</math>
              ;;
              (""       "<math>1+2</math>"       ""       "1+2")
              (""       "<math>1\n+\n2</math>"   ""       "1\n+\n2")
              ("$$$"    "<math>1+2</math>"       "$$$"    "1+2")
              ("@math{" "<math>1+2</math>"       "}}"     "1+2")
              ("@math{@math{" "<math>1+2</math>" "}}"     "1+2")

              ;; @math{...}
              ;;
              (""       "@math{1+2}"      ""          "1+2")
              (""       "@math{\n1+2\n}"  ""          "\n1+2\n")
              ("$"      "@math{1+2}"      "$"         "1+2")
              ("<math>" "@math{1+2}"      "</math>"   "1+2")

              ;; <alt role="tex">$...$</alt>
              ;;
              ("<alt role=\"tex\">" "$C=\\alpha" "$</alt>"         "C=\\alpha")
              (""  "<alt role=\"tex\">$C=\\alpha$</alt>" ""        "C=\\alpha")
              ("$" "<alt role=\"tex\">$C=\\alpha$</alt>" ""        "C=\\alpha")
              ("$" "<alt role=\"tex\">$C=\\alpha$"       "</alt>$" "C=\\alpha")
              (""  "<alt\n \trole=\"tex\">$C=\\alpha$</alt>" ""    "C=\\alpha")

              ;; \(...\)
              ;;
              ("" "\\(1+2\\)"     ""  "\\(1+2\\)")
              ("" "\\(1\n+\n2\\)" ""  "\\(1\n+\n2\\)")
              ("" "\\(X(i)+2\\)"  ""  "\\(X(i)+2\\)")
              ("" "\\( \\\\ \\)"  ""  "\\( \\\\ \\)")
              ;; ("" "\\( (\\\\) \\)"  ""  "\\( (\\\\) \\)")

              ;; \[...\]
              ;;
              ("" "\\[1+2\\]"     ""  "\\[1+2\\]")
              ("" "\\[1\n+\n2\\]" ""  "\\[1\n+\n2\\]")
              ("" "\\[X[i]+2\\]"  ""  "\\[X[i]+2\\]")
              ("" "\\[ \\\\ \\]"  ""  "\\[ \\\\ \\]")
              ;; ("" "\\[ [\\\\] \\]"  ""  "\\[ [\\\\] \\]")

              ;; [;...;]
              ;;
              ("" "[;1+2;]"     ""  "[;1+2;]")
              ("" "[;1\n+\n2;]" ""  "[;1\n+\n2;]")
              ("" "[;X[i]+2;]"  ""  "[;X[i]+2;]")
              ("" "[; \\\\ ;]"  ""  "[; \\\\ ;]")

              ;; \begin{math}...\end{math}
              ;;
              ("" "\\begin{math}1+2\\end{math}"     ""
               "\\begin{math}1+2\\end{math}")
              ("" "\\begin{math}1\n+\n2\\end{math}" ""
               "\\begin{math}1\n+\n2\\end{math}")
              ("" "\\begin{math}X(i)+2\\end{math}"  ""
               "\\begin{math}X(i)+2\\end{math}")
              ("" "\\begin{math} \\\\ \\end{math}"  ""
               "\\begin{math} \\\\ \\end{math}")
              ("" "\\begin{math} (foo) \\end{math}" ""
               "\\begin{math} (foo) \\end{math}")

              ;; \begin{displaymath}...\end{displaymath}
              ;;
              ("" "\\begin{displaymath}1+2\\end{displaymath}"     ""
               "\\begin{displaymath}1+2\\end{displaymath}")
              ("" "\\begin{displaymath}1\n+\n2\\end{displaymath}" ""
               "\\begin{displaymath}1\n+\n2\\end{displaymath}")
              ("" "\\begin{displaymath}X(i)+2\\end{displaymath}"  ""
               "\\begin{displaymath}X(i)+2\\end{displaymath}")
              ("" "\\begin{displaymath} \\\\ \\end{displaymath}"  ""
               "\\begin{displaymath} \\\\ \\end{displaymath}")
              ("" "\\begin{displaymath} (foo) \\end{displaymath}" ""
               "\\begin{displaymath} (foo) \\end{displaymath}")

              ;; \begin{equation}...\end{equation}
              ;;
              ("" "\\begin{equation}1+2\\end{equation}"     ""
               "\\begin{equation}1+2\\end{equation}")

              ;; \begin{align}...\end{align}
              ;;
              ("" "\\begin{align}1+2\\end{align}"     ""
               "\\begin{align}1+2\\end{align}")
))

    ;;-------------------------------------------------------------------------
    ;; org-mode

    ;; (if (not (fboundp 'org-mode))
    ;;     (message "skip, no org-mode to exercise")
    ;;
    ;;   (let ((str (concat (org-latex-packages-to-string
    ;;                       org-export-latex-default-packages-alist)
    ;;                      (org-latex-packages-to-string
    ;;                       org-export-latex-packages-alist)))
    ;;
    ;; (fundamental-mode)

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (let ((feature 'tex-math-preview)
          (prop    '(tex-math bounds-of-thing-at-point)))
      (unless (apply 'get prop)
        (error "oops, didn't find thing-at-point property"))

      (unload-feature feature t)

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)
      (if (not my-unload-feature-function-p)
          (message "skip, no unload-feature function")

        (let ((func (apply 'get prop)))
          (when func
            (error "unload-feature leftover %S property: %S" prop func)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
